/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wPardiso.h"
#ifdef USE_MKL

using namespace tbox;

void Pardiso::analyze(Eigen::SparseMatrix<double> const &A)
{
    solver.analyzePattern(A);
}
void Pardiso::factorize(Eigen::SparseMatrix<double> const &A)
{
    solver.factorize(A);
}
void Pardiso::solve(Eigen::Map<Eigen::VectorXd> const &b, Eigen::Map<Eigen::VectorXd> &x)
{
    x = solver.solve(b);
}
void Pardiso::compute(Eigen::SparseMatrix<double> const &A, Eigen::Map<Eigen::VectorXd> const &b, Eigen::Map<Eigen::VectorXd> &x)
{
    solver.compute(A);
    x = solver.solve(b);
}

/**
 * @brief Set option for Pardiso
 */
void Pardiso::setOption(int k, int v)
{
    if (k >= 0)
        solver.pardisoParameterArray()[k] = v;
    else
        throw std::runtime_error("tbox::Pardiso::setOption: option id 'k' must be positive!\n");
}

void Pardiso::write(std::ostream &out) const
{
    out << "Pardiso (MKL/Eigen)" << std::endl;
}

#endif // USE_MKL
